@@ -1,8 +1,9 @@
 package CGI::Widget::DBI::Search::Base;
 
-require 5.004;
 use strict;
 
+use Scalar::Util qw/blessed/;
+
 # --------------------- USER CUSTOMIZABLE VARIABLES ------------------------
 
 use constant DEBUG => 0;
@@ -12,19 +13,24 @@ use constant DEBUG => 0;
 sub new {
     my $this = shift;
     my $class = ref($this) || $this;
-    my $self = bless { @_ }, $class;
+    my $self = ref($_[0]) =~ m/^CGI::Widget::DBI::/ && scalar(@_) == 1
+      ? bless { %{ $_[0] } }, $class
+      : bless { @_ }, $class;
     $self->initialize if $self->can('initialize');
     return $self;
 }
 
-###############################################################################
-# Apache logging methods
-###############################################################################
+sub caller_function {
+    my ($self, $stacklvl) = @_;
+    my ($func) = ( (caller($stacklvl || 1))[3] =~ m/::([^:]+)\z/ );
+    return $func || '';
+}
 
 sub log_error {
-    my ($self, $method, $msg) = @_;
+    my ($self, $msg) = @_;
+    my $method = $self->caller_function(2) || $self->caller_function(3); # check one level higher in case called from eval
     my $logmsg = (ref($self)||$self)."->".$method.": ".$msg;
-    if (ref $self->{r} eq "Apache") {
+    if (blessed($self->{r}) && $self->{r}->can('log_error')) {
 	$self->{r}->log_error($logmsg);
     } elsif (ref $self->{parent} and ref $self->{parent}->{r} eq "Apache") {
 	$self->{parent}->{r}->log_error($logmsg);
@@ -35,10 +41,11 @@ sub log_error {
 }
 
 sub warn {
-    my ($self, $method, $msg) = @_;
+    my ($self, $msg) = @_;
     return unless $self->{_DEBUG} || DEBUG;
+    my $method = $self->caller_function(2) || $self->caller_function(3);
     my $logmsg = (ref($self)||$self)."->".$method.": ".$msg;
-    if (ref $self->{r} eq "Apache") {
+    if (blessed($self->{r}) && $self->{r}->can('warn')) {
 	$self->{r}->warn($logmsg);
     } elsif (ref $self->{parent} and ref $self->{parent}->{r} eq "Apache") {
 	$self->{parent}->{r}->warn($logmsg);
@@ -56,27 +63,4 @@ __END__
 
 Adi Fairbank <adi@adiraj.org>
 
-=head1 COPYRIGHT
-
-Copyright (c) 2004-2008 - Adi Fairbank
-
-This software, the CGI::Widget::DBI::Search::Base Perl module,
-is copyright Adi Fairbank.
-
-=head1 LICENSE
-
-This module is free software; you can redistribute it and/or modify it
-under the terms of either:
-
-  a) the GNU General Public License as published by the Free Software
-     Foundation; either version 1, or (at your option) any later version,
-
-  or
-
-  b) the "Artistic License" which comes with this module.
-
-=head1 LAST MODIFIED
-
-April 9, 2008
-
 =cut
